#!BPY

"""
Name: 'Bezier-Curve Triangle Fill'
Blender: 237
Group: 'Object'
Tooltip: 'fills a plane bezier-curve object with triangles'
"""
__author__ = "Peter Baumann"
__url__ = ("blender", "elysiun")
__version__ = "1.0"


# define the needed path for the external Program here:
#======================================================
TrianglePrgPath = 'C:\\Programme\\Triangle\\triangle.exe'

# define the path and name of the temporary poly-file here:
#==========================================================
PolyFileName = 'C:\\temp\\TriangleMesh.poly'

##########################################################
#GUI Created using RipSting's Blender-Python GUI designer#
#Download at Http://oregonstate.edu/~dennisa/Blender/BPG/#
##########################################################

import Blender, meshtools
import os
import sys
import math
from Blender.BGL import *
from Blender.Draw import *
from Blender.Noise import *

Area = Create(1.0)
#DefHolesMode = Create(0)
#UseHoles = Create(0)

def draw():
	global Area, DefHolesMode, UseHoles, ObjectName, TriangulateButton, ExitButton, ResetHoles

	glClearColor(0.753, 0.753, 0.753, 0.0)
	glClear(GL_COLOR_BUFFER_BIT)


	glColor3f(0.000, 0.000, 0.000)
	glRasterPos2i(120, 108)
	Text('Max Area:')


	Button('Triangulate', 1, 8, 85, 100, 30, '')
	Button('Exit', 2, 230, 84, 100, 31, '')
	Button('Reset Holes', 3, 230, 45, 100, 30, '')
#	Button('Reset Holes', 4, 120, 45, 100, 30, '')
	Button('Add Hole', 5, 8, 44, 100, 30, '')

#	ObjectName = String('', 4, 120, 80, 100, 20, ObjectName.val, 512, '')

#	DefHolesMode = Toggle('Define Holes Mode', 5, 120, 45, 100, 30, DefHolesMode.val, '')
#	UseHoles = Toggle('Use Holes', 6, 8, 44, 100, 31, UseHoles.val, '')


	Area = Number('', 7, 120, 84, 100, 15, Area.val, 0.00001, 1000, '')

def event(evt, val):
	if (evt== QKEY and not val): Exit()
def bevent(evt):
	if evt == 1: #TriangulateButton
		makeMesh()

	elif evt == 2: #ExitButton
		print "That's it ... bye!"
		for ob in hole_obs:     # delete all hole-objects
			if ob.name != "":
				scene = Blender.Scene.getCurrent()
				scene.unlink(ob)
		freeHoleList(hole_obs)
		Exit()

	elif evt == 3: #ResetHoles
		print "Reset Holes"
		for ob in hole_obs:     # delete all hole-objects
			if ob.name != "":
				scene = Blender.Scene.getCurrent()
				scene.unlink(ob)
		freeHoleList(hole_obs)  # free the hole-list

#	elif evt == 4: #String ObjectName
#		print "Object Name: ", ObjectName.val

	elif evt == 5: # AddHole
		print "Add Hole"
		hx, hy, hz = Blender.Window.GetCursorPos()
		HoleOb = Blender.Object.New ('Empty')
		HoleOb.setName('hole%i' % (len(hole_obs)+1))
		HoleOb.setLocation(hx, hy, 0)
		scene = Blender.Scene.getCurrent()
		scene.link(HoleOb)
		hole_obs.append(HoleOb)

#	elif evt == 6: #UseHoles
#		print "UseHoles: ", UseHoles.val

	elif evt == 7: #AreaNumber
		print "Area: ", Area.val

	Blender.Redraw()

#================#
# Error Message  #
#================#
def error(str):
	popup = Blender.Draw.PupMenu('ERROR:%t|'+str+'|Click to Return')

#=====================#
# Free list of curves #
#=====================#
def freeCurveList(bezier_curve_obs):
	listIndex = len(bezier_curve_obs) # free curve list
	while listIndex:
		listIndex -=1
		bezier_curve_obs.pop(listIndex)

#=====================#
# Free lists of holes #
#=====================#
def freeHoleList(hole_obs):
	listIndex = len(hole_obs) # free list of Hole X-values
	while listIndex:
		listIndex -=1
		hole_obs.pop(listIndex)

# =============================
# ====== Read OFF Format ======  Copied from off_import.py
# =============================  Many thanks to Anthony!
#__author__ = "Anthony D'Agostino (Scorpius)"
#__url__ = ("blender", "elysiun",
#"Author's homepage, http://www.redrival.com/scorpius")
#__version__ = "Part of IOSuite 0.5"

def read(filename):
	file = open(filename, "rb")

	verts = []
	faces = []
	uv = []

	# === OFF Header ===
	offheader = file.readline()
	numverts, numfaces, null = file.readline().split()
	numverts = int(numverts)
	numfaces = int(numfaces)
	if offheader.find('ST') >= 0:
		has_uv = True
	else:
		has_uv = False

	# === Vertex List ===
	for i in range(numverts):
		if not i%100 and meshtools.show_progress:
			Blender.Window.DrawProgressBar(float(i)/numverts, "Reading Verts")
		if has_uv:
			x, y, z, u, v = map(float, file.readline().split())
			uv.append((u, v))
		else:
			x, y, z = map(float, file.readline().split())
		verts.append((x, y, z))

	# === Face List ===
	for i in range(numfaces):
		if not i%100 and meshtools.show_progress:
			Blender.Window.DrawProgressBar(float(i)/numfaces, "Reading Faces")
		line = file.readline().split()
		numfaceverts = len(line)-1
		facev = []
		for j in range(numfaceverts):
			index = int(line[j+1])
			facev.append(index)
		facev.reverse()
		faces.append(facev)

	objname = Blender.sys.splitext(Blender.sys.basename(filename))[0]

	meshtools.create_mesh(verts, faces, objname, faces, uv)
	Blender.Window.DrawProgressBar(1.0, '')  # clear progressbar
	file.close()

#==============================#
# fills the curve with a mesh  #
#==============================#
def makeMesh():
	time1 = Blender.sys.time()
	curveObjects = 0
	fileContainsCurve = False
	for ob in Blender.Object.GetSelected():
		if 'Curve' == ob.getType():
			bezier_curve_obs.append(ob)
			curveObjects +=1
	if curveObjects == 0:
		print "No curve selected"
		error("Please select a closed Bezier curve")
		freeCurveList(bezier_curve_obs)
		return
	curves = []
	locations = []
	Index = 0
	BackupIndex = 0
	NodesAndEdges = 0
	file = open(PolyFileName, "w")
	std = sys.stdout
	sys.stdout = file
	print "# Poly-File created by Blender (www.blender.org)"
	Blender.Window.WaitCursor(1) # to let user knowns the script is busy.
	print "# Create Vertices of all selected curves:"
	for object in bezier_curve_obs:  # for later versions -> more than one curve will define holes
		ob = object.getData() # get points of the curve
		if len(ob[0]) > 2: # curve must have at least 3 Points for a closed loop
			NodesAndEdges += len(ob[0])
	print "# No. of Points: ", NodesAndEdges
	print NodesAndEdges, "2 0 0"     # Info about no. of points, 2D, no attributes, no boundary marker
	for object in bezier_curve_obs:  # fsupport for more than one curve
		ob = object.getData() # get points of the curve
		x, y, z = object.getLocation() # Location of curve (only x/y is used, z will be ignored)
		ob_rot = object.getEuler()     # Rotation of the curve
		sx, sy, sz = object.getSize()  # Size of the curve
		#----------------------
		# Create the .poly-file
		#----------------------
		if len(ob[0]) > 2: # curve must have at least 3 Points for a closed loop
			fileContainsCurve = True
			print "# "+ob.name
			print "# List of Vertices :"
			for i in range(0, len(ob[0])):
				xh1, yh1, zh1, xpt, ypt, zpt, xh2, yh2, zh2 = ob.getControlPoint(0,i)
				print Index+1, ' ', math.cos(-ob_rot.z)*(xpt*sx)+math.sin(-ob_rot.z)*(ypt*sy)+x, math.sin(ob_rot.z)*(xpt*sx)+math.cos(ob_rot.z)*(ypt*sy)+y    # 2D Point in X/Y plane (ignore Z-value)
				Index +=1
	Index = 0
	print "# Create Boundary Edges of all selected Curves:"
	print "# No. of Edges: ", NodesAndEdges
	print NodesAndEdges, "0"     # Info about no. of boundary edges
	for object in bezier_curve_obs:
		ob = object.getData() # get points of the curve
		if len(ob[0]) > 2: # curve must have at least 3 Points for a closed loop
			fileContainsCurve = True
			print "# "+ob.name
			for i in range(0, len(ob[0])-1):
				print Index+1, ' ', Index+1, Index+2
				Index +=1
			print Index+1, ' ', Index+1, BackupIndex+1
			Index +=1
		BackupIndex = Index
	Index = 0
	print "# No. of holes", len(hole_obs)
	print len(hole_obs)         # Info about no. of holes
	for hole in hole_obs:
		hx, hy, hz = hole.getLocation()   # get list of created holes
		print Index+1, ' ', hx, hy

	sys.stdout = std
	file.close()
	#---------------------------
	# triangulate the .poly-file
	#---------------------------
	triangle = TrianglePrgPath
	arg1 = "-q " # produce quality mesh (avoid small angles of triangles)
	arg2 = "-g " # create an Object File Format (.off) of the mesh
	arg3 = "-P -N -E " # suppress the .poly, .node and .ele files -> not needed
	arg4 = "-a%f " % Area.val  # defines the max Area for each triangle
	arg5 = "-V "  # Enables verbous mode -> more infos about the created mesh
	command = triangle+" "+arg1+arg2+arg3+arg4+arg5+PolyFileName
	if fileContainsCurve:
		os.system(command)  # execute triangle and let it do the meshing job
		#-----------------------------------------
		# Read in the created .off-file
		#-----------------------------------------
		offfilename = PolyFileName[:-5]+".1.off"
		read(offfilename)
	freeCurveList(bezier_curve_obs)
#	freeHoleList(hole_obs)
	Blender.Window.WaitCursor(0) # Return Cursor to default.
	Blender.Window.RedrawAll()
#	Blender.Redraw()
	print "Used time: ", Blender.sys.time() - time1

#============================#
# Main Program:              #
# Register the GUI and start #
#============================#

bezier_curve_obs = []
hole_obs = []

Register(draw, event, bevent)

